
function BHDragableController(){
    this._containers = new Array();
    this._floater = null;
    this._landingCallBack = null;
    this._inDrag = null;
    this._takeoff = null;
    this._landing = null;
    this._originalBackgroundColor = "";
    this._originalX = "";
    this._cursorX = "";
    this._originalY = "";
    this._cursorY = "";
    
    document.addEventListener("mousedown", createMethodReference(this, "dragInitiated"), true);
    document.addEventListener("mousemove", createMethodReference(this, "dragging"), true);
    document.addEventListener("mouseup", createMethodReference(this, "dropped"), true);
    
    ////////////////////////////////////////////
    
    this.setLandingCallback = function(c){
        this._landingCallBack = c;
    }
    
    this.addContainer = function(container){
        this._containers.push(container);
    }
    
    this.screenPosition = function(pt){
        var r = {x:0,y:0};
        while(pt.offsetParent){
            r.x += pt.offsetLeft - pt.scrollLeft; r.y += pt.offsetTop - pt.scrollTop;
            pt = pt.offsetParent;
        }
        return r;
    }
    
    this.cloneFloater = function(){
        
        this._floater = this._inDrag.cloneNode(true);
        
        this._floater.style.position = "absolute";
        this._floater.style.listStyle = "none";
        this._floater.style.zIndex = 10002;
        
        document.body.appendChild(this._floater);
        
        var p = this.screenPosition(this._inDrag);
        
        this._floater.style.left = p.x + "px"; this._floater.style.top = p.y + "px";
        
        this._originalX = p.x; this._originalY = p.y;
        this._cursorX = event.clientX; this._cursorY = event.clientY;
        this._floater.style.width = this._inDrag.clientWidth;this._floater.style.height = this._inDrag.clientHeight;
        this._inDrag = this._floater;
    }
    
    this.dragInitiated = function(){
        for(var i = 0; i < this._containers.length; i++){
            if (this._containers[i].objectIsADragon(event.target)){
                this._inDrag = event.target;
                this.cloneFloater();
                this.makeUpOn();
                this._takeoff = this._containers[i];
                event.preventDefault();    
            }
        }
    }
    
    this.makeUpOn = function(){
        this._inDrag.style.opacity = "0.75";
        this._inDrag.style.backgroundColor = "#C6D8E8";
        this._inDrag.style.border = "1px dotted #F1F1F1";
    }
    
    this.dragging = function(){
        if (!this._inDrag) return;
        for(var i = 0; i < this._containers.length; i++){this._containers[i].landingLights();}
        this._inDrag.style.left = (event.clientX - this._cursorX) + this._originalX + "px";
        this._inDrag.style.top = (event.clientY - this._cursorY) + this._originalY + "px";
    }
    
    this.dropped = function(){
        if (!this._inDrag) return;
        for(var i = 0; i < this._containers.length; i++){
            if (this._containers[i].isInside()) this._landing = this._containers[i];
            this._containers[i].clearLandingLights();
        }
        
        if (this._landing && this._landingCallBack) this._landingCallBack();
        
        this._inDrag = null;
        this._takeoff= null;
        this._landing = null;
        document.body.removeChild(this._floater);
        this._floater = null;
    }
    
    this.destroy = function(){
        this._containers.clear();
        document.removeEventListener("mousedown", createMethodReference(this, "dragInitiated"), true);
        document.removeEventListener("mousemove", createMethodReference(this, "dragging"), true);
        document.removeEventListener("mouseup", createMethodReference(this, "dropped"), true);        
    }
    
}


function BHDragableContainer(containerObject, controller){ 
    this._container = containerObject;
    this._controller = controller;   
    this._dragons = new Array();
    this._originalBackground = containerObject.style.backgroundColor;
    this._originalBorderBottomColor = containerObject.style.borderBottomColor;
    this._originalBorderBottomStyle = containerObject.style.borderBottomStyle;
    this._originalBorderBottomWidth = containerObject.style.borderBottomWidth;
    ////
    this._originalBorderLeftColor = containerObject.style.borderLeftColor;
    this._originalBorderLeftStyle = containerObject.style.borderLeftStyle;
    this._originalBorderLeftWidth = containerObject.style.borderLeftWidth;
    ///
    this._originalBorderTopColor = containerObject.style.borderTopColor;
    this._originalBorderTopStyle = containerObject.style.borderTopStyle;
    this._originalBorderTopWidth = containerObject.style.borderTopWidth;  
    ///
    this._originalBorderRightColor = containerObject.style.borderRightColor;
    this._originalBorderRightStyle = containerObject.style.borderRightStyle;
    this._originalBorderRightWidth = containerObject.style.borderRightWidth;
    
    this._lightsOn = false;
    
    this._containerCords = absolutePosition(this._container);
    
    var ds = new Array();
    ds.addNodes(containerObject.getElementsByTagName("li"));
    ds.addNodes(containerObject.getElementsByTagName("div"));
  
    for (var i = 0; i < ds.length; i++){
        ds[i].style.cursor = "move";
        this._dragons.push(ds[i]);
    }
    
    controller.addContainer(this);
    
    ///////////////////////////////////////////////////////////////
    
    this.huntForDragons = function(){
        this._dragons.clear();
        var ds = new Array();
        ds.addNodes(containerObject.getElementsByTagName("li"));
        ds.addNodes(containerObject.getElementsByTagName("div"));
    
        for (var i = 0; i < ds.length; i++){
            ds[i].style.cursor = "move";
            this._dragons.push(ds[i]);
        }
    }
    
    this.isInside = function(){
        var p = absolutePosition(this._container);
        
        if ( (event.clientX > p.x) && (event.clientX < p.x + this._container.clientWidth) && (event.clientY > p.y) && (event.clientY < p.y + this._container.clientHeight)    ) return true;
        return false;
    }
    
    this.landingLights = function(){
        if (this.isInside()){
            if (!this._lightsOn) this.flashLandingLights();
        }else{
            if (this._lightsOn) this.clearLandingLights();
        }
    }
    
    this.flashLandingLights = function(){
        bc = "#326FBC"; bs = "solid"; bw = "1px";
        this._container.style.backgroundColor = "#B2C2D5";
        this._container.style.borderBottomColor = bc;
        this._container.style.borderBottomStyle = bs;
        this._container.style.borderBottomWidth = bw;
        ////
        this._container.style.borderLeftColor = bc;
        this._container.style.borderLeftStyle = bs;
        this._container.style.borderLeftWidth = bw;
        ////
        this._container.style.borderTopColor = bc;
        this._container.style.borderTopStyle = bs;
        this._container.style.borderTopWidth = bw;
        ////
        this._container.style.borderRightColor = bc;
        this._container.style.borderRightStyle = bs;
        this._container.style.borderRightWidth = bw;
            
        this._lightsOn = true;
    }
    
    this.clearLandingLights = function(){
        this._container.style.backgroundColor = this._originalBackground;
        
        this._container.style.borderBottomColor = this._originalBorderBottomColor;
        this._container.style.borderBottomStyle = this._originalBorderBottomStyle;
        this._container.style.borderBottomWidth = this._originalBorderBottomWidth;
        ////
        this._container.style.borderLeftColor = this._originalBorderLeftColor;
        this._container.style.borderLeftStyle = this._originalBorderLeftStyle;
        this._container.style.borderLeftWidth = this._originalBorderLeftWidth;       
        ////
        this._container.style.borderTopColor = this._originalBorderTopColor;
        this._container.style.borderTopStyle = this._originalBorderTopStyle;
        this._container.style.borderTopWidth = this._originalBorderTopWidth; 
        ////
        this._container.style.borderRightColor = this._originalBorderRightColor;
        this._container.style.borderRightStyle = this._originalBorderRightStyle;
        this._container.style.borderRightWidth = this._originalBorderRightWidth; 
        
        this._lightsOn = false;
    }
    
    this.objectIsADragon = function(x){
        for(var i = 0; i < this._dragons.length; i++){
            if (x == this._dragons[i]) return true;
        }
        return false;
    }
    
    this.numberOfDragons = function(){return this._dragons.length;}
}
